@@ -111,6 +111,10 @@ struct FMListViewDetails {
 	GtkWidget *column_editor;
 
 	char *original_name;
+	
+	NautilusFile *renaming_file;
+	gboolean rename_done;
+	guint renaming_file_activate_timeout;
 };
 
 struct SelectionForeachData {
@@ -128,12 +132,16 @@ struct SelectionForeachData {
 /* We wait two seconds after row is collapsed to unload the subdirectory */
 #define COLLAPSE_TO_UNLOAD_DELAY 2000 
 
+/* Wait for the rename to end when activating a file being renamed */
+#define WAIT_FOR_RENAME_ON_ACTIVATE 200
+
 static int                      click_policy_auto_value;
 static char *              	default_sort_order_auto_value;
 static gboolean			default_sort_reversed_auto_value;
 static NautilusZoomLevel        default_zoom_level_auto_value;
 static GList *                  default_visible_columns_auto_value;
 static GList *                  default_column_order_auto_value;
+static GdkCursor *              hand_cursor = NULL;
 
 static GList *fm_list_view_get_selection                   (FMDirectoryView   *view);
 static GList *fm_list_view_get_selection_for_file_transfer (FMDirectoryView   *view);
@@ -145,7 +153,9 @@ static void   fm_list_view_scale_font_size                 (FMListView        *v
 static void   fm_list_view_scroll_to_file                  (FMListView        *view,
 							    NautilusFile      *file);
 static void   fm_list_view_iface_init                      (NautilusViewIface *iface);
-
+static void   fm_list_view_rename_callback                 (NautilusFile      *file,
+							    GnomeVFSResult     result,
+							    gpointer           callback_data);
 
 
 G_DEFINE_TYPE_WITH_CODE (FMListView, fm_list_view, FM_TYPE_DIRECTORY_VIEW, 
@@ -200,6 +210,23 @@ activate_selected_items (FMListView *view)
 	GList *file_list;
 	
 	file_list = fm_list_view_get_selection (FM_DIRECTORY_VIEW (view));
+
+	
+	if (view->details->renaming_file) {
+		/* We're currently renaming a file, wait until the rename is
+		   finished, or the activation uri will be wrong */
+		if (view->details->renaming_file_activate_timeout == 0) {
+			view->details->renaming_file_activate_timeout =
+				g_timeout_add (WAIT_FOR_RENAME_ON_ACTIVATE, (GSourceFunc) activate_selected_items, view);
+		}
+		return;
+	}
+	
+	if (view->details->renaming_file_activate_timeout != 0) {
+		g_source_remove (view->details->renaming_file_activate_timeout);
+		view->details->renaming_file_activate_timeout = 0;
+	}
+	
 	fm_directory_view_activate_files (FM_DIRECTORY_VIEW (view),
 					  file_list,
 					  NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
@@ -431,9 +458,6 @@ motion_notify_callback (GtkWidget *widget,
 {
 	FMListView *view;
 	GdkDragContext *context;
-	GdkCursor *cursor;
-	GtkTreePath *last_hover_path;
-	GtkTreeIter iter;
 	
 	view = FM_LIST_VIEW (callback_data);
 	
@@ -442,42 +466,25 @@ motion_notify_callback (GtkWidget *widget,
 	}
 
 	if (click_policy_auto_value == NAUTILUS_CLICK_POLICY_SINGLE) {
-		last_hover_path = view->details->hover_path;
+		GtkTreePath *old_hover_path;
 
+		old_hover_path = view->details->hover_path;
 		gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget),
 					       event->x, event->y,
 					       &view->details->hover_path,
 					       NULL, NULL, NULL);
 
-		if (view->details->hover_path != NULL) {
-			cursor = gdk_cursor_new (GDK_HAND2);
-		} else {
-			cursor = NULL;
-		}
-
-		gdk_window_set_cursor (widget->window, cursor);
-
-		/* only redraw if the hover row has changed */
-		if (!(last_hover_path == NULL && view->details->hover_path == NULL) &&
-		    (!(last_hover_path != NULL && view->details->hover_path != NULL) ||
-		     gtk_tree_path_compare (last_hover_path, view->details->hover_path))) {
-			if (last_hover_path) {
-				if (gtk_tree_model_get_iter (GTK_TREE_MODEL (view->details->model),
-							     &iter, last_hover_path)) {
-					gtk_tree_model_row_changed (GTK_TREE_MODEL (view->details->model),
-								    last_hover_path, &iter);
-				}
-			}
-
-			if (view->details->hover_path) {
-				gtk_tree_model_get_iter (GTK_TREE_MODEL (view->details->model),
-							 &iter, view->details->hover_path);
-				gtk_tree_model_row_changed (GTK_TREE_MODEL (view->details->model),
-							    view->details->hover_path, &iter);
+		if ((old_hover_path != NULL) != (view->details->hover_path != NULL)) {
+			if (view->details->hover_path != NULL) {
+				gdk_window_set_cursor (widget->window, hand_cursor);
+			} else {
+				gdk_window_set_cursor (widget->window, NULL);
 			}
 		}
 
-		gtk_tree_path_free (last_hover_path);
+		if (old_hover_path != NULL) {
+			gtk_tree_path_free (old_hover_path);
+		}
 	}
 
 	if (view->details->drag_button != 0) {
@@ -505,22 +512,40 @@ leave_notify_callback (GtkWidget *widget,
 		       gpointer callback_data)
 {
 	FMListView *view;
-	GtkTreeIter iter;
 
 	view = FM_LIST_VIEW (callback_data);
 
 	if (click_policy_auto_value == NAUTILUS_CLICK_POLICY_SINGLE &&
 	    view->details->hover_path != NULL) {
-		if (gtk_tree_model_get_iter (GTK_TREE_MODEL (view->details->model),
-					     &iter, view->details->hover_path)) {
-			gtk_tree_model_row_changed (GTK_TREE_MODEL (view->details->model),
-						    view->details->hover_path, &iter);
-		}
-
 		gtk_tree_path_free (view->details->hover_path);
 		view->details->hover_path = NULL;
+	}
 
-		return TRUE;
+	return FALSE;
+}
+
+static gboolean
+enter_notify_callback (GtkWidget *widget,
+		       GdkEventCrossing *event,
+		       gpointer callback_data)
+{
+	FMListView *view;
+
+	view = FM_LIST_VIEW (callback_data);
+
+	if (click_policy_auto_value == NAUTILUS_CLICK_POLICY_SINGLE) {
+		if (view->details->hover_path != NULL) {
+			gtk_tree_path_free (view->details->hover_path);
+		}
+
+		gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget),
+					       event->x, event->y,
+					       &view->details->hover_path,
+					       NULL, NULL, NULL);
+
+		if (view->details->hover_path != NULL) {
+			gdk_window_set_cursor (widget->window, hand_cursor);
+		}
 	}
 
 	return FALSE;
@@ -977,7 +1002,9 @@ cell_renderer_edited (GtkCellRendererText *cell,
 
 	/* Only rename if name actually changed */
 	if (strcmp (new_text, view->details->original_name) != 0) {
-		fm_rename_file (file, new_text);
+		view->details->renaming_file = nautilus_file_ref (file);
+		view->details->rename_done = FALSE;
+		fm_rename_file (file, new_text, fm_list_view_rename_callback, g_object_ref (view));
 		g_free (view->details->original_name);
 		view->details->original_name = g_strdup (new_text);
 	}
@@ -1059,57 +1086,57 @@ move_copy_items_callback (NautilusTreeViewDragDest *dest,
 }
 
 static void
-apply_visible_columns_foreach (gpointer key, gpointer value, gpointer user_data)
+apply_columns_settings (FMListView *list_view, GList *column_order, GList *visible_columns)
 {
-	gboolean visible;
-	
-	visible = (eel_g_str_list_index (user_data, key) != -1);
-	
-	gtk_tree_view_column_set_visible (GTK_TREE_VIEW_COLUMN (value), 
-					  visible);
-}
-
-static void
-apply_visible_columns (FMListView *list_view, GList *visible_columns)
-{
-	g_hash_table_foreach (list_view->details->columns, 
-			      apply_visible_columns_foreach,
-			      visible_columns);
-	
-}
-
-static void
-apply_column_order (FMListView *list_view, GList *column_order)
-{
-	GList *columns;	
-	GtkTreeViewColumn *last_view_column;
+	GList *all_columns;
+	GList *old_view_columns, *view_columns;
+	GtkTreeViewColumn *prev_view_column;
 	GList *l;
-	
-	columns = nautilus_get_all_columns ();
-	columns = nautilus_sort_columns (columns, column_order);
 
-	last_view_column = NULL;
-	for (l = columns; l != NULL; l = l->next) {
-		GtkTreeViewColumn *view_column;
+	/* prepare ordered list of view columns using column_order and visible_columns */
+	view_columns = NULL;
+	all_columns = nautilus_get_all_columns ();
+	all_columns = nautilus_sort_columns (all_columns, column_order);
+	for (l = all_columns; l != NULL; l = l->next) {
 		char *name;
 
 		g_object_get (G_OBJECT (l->data), "name", &name, NULL);
-		
-		view_column = g_hash_table_lookup (list_view->details->columns,
-						   name);
+		if (g_list_find_custom (visible_columns, name, (GCompareFunc) g_ascii_strcasecmp) != NULL) {
+			GtkTreeViewColumn *view_column;
 
+			view_column = g_hash_table_lookup (list_view->details->columns, name);
+			if (view_column != NULL) {
+				view_columns = g_list_append (view_columns, view_column);
+			}
+		}
 		g_free (name);
-		
-		if (view_column) {
-			gtk_tree_view_move_column_after 
-				(list_view->details->tree_view,
-				 view_column,
-				 last_view_column);
-			
-			last_view_column = view_column;
+	}
+	nautilus_column_list_free (all_columns);
+
+	/* remove columns that are not present in the configuration */
+	old_view_columns = gtk_tree_view_get_columns (list_view->details->tree_view);
+	for (l = old_view_columns; l != NULL; l = l->next) {
+		if (g_list_find (view_columns, l->data) == NULL) {
+			gtk_tree_view_remove_column (list_view->details->tree_view, l->data);
 		}
 	}
-	nautilus_column_list_free (columns);
+	g_list_free (old_view_columns);
+
+	/* append new columns from the configuration */
+	old_view_columns = gtk_tree_view_get_columns (list_view->details->tree_view);
+	for (l = view_columns; l != NULL; l = l->next) {
+		if (g_list_find (old_view_columns, l->data) == NULL) {
+			gtk_tree_view_append_column (list_view->details->tree_view, l->data);
+		}
+	}
+	g_list_free (old_view_columns);
+
+	/* place columns in the correct order */
+	prev_view_column = NULL;
+	for (l = view_columns; l != NULL; l = l->next) {
+		gtk_tree_view_move_column_after (list_view->details->tree_view, l->data, prev_view_column);
+		prev_view_column = l->data;
+	}
 }
 
 static void
@@ -1165,7 +1192,7 @@ create_and_set_up_tree_view (FMListView *view)
 	view->details->columns = g_hash_table_new_full (g_str_hash, 
 							g_str_equal,
 							(GDestroyNotify)g_free,
-							NULL);
+							(GDestroyNotify) g_object_unref);
 	gtk_tree_view_set_enable_search (view->details->tree_view, TRUE);
 
 	/* Don't handle backspace key. It's used to open the parent folder. */
@@ -1206,6 +1233,8 @@ create_and_set_up_tree_view (FMListView *view)
 				 G_CALLBACK (drag_data_get_callback), view, 0);
 	g_signal_connect_object (view->details->tree_view, "motion_notify_event",
 				 G_CALLBACK (motion_notify_callback), view, 0);
+	g_signal_connect_object (view->details->tree_view, "enter_notify_event",
+				 G_CALLBACK (enter_notify_callback), view, 0);
 	g_signal_connect_object (view->details->tree_view, "leave_notify_event",
 				 G_CALLBACK (leave_notify_callback), view, 0);
 	g_signal_connect_object (view->details->tree_view, "button_press_event",
@@ -1267,6 +1296,8 @@ create_and_set_up_tree_view (FMListView *view)
 			view->details->pixbuf_cell = (GtkCellRendererPixbuf *)cell;
 			
 			view->details->file_name_column = gtk_tree_view_column_new ();
+			g_object_ref (view->details->file_name_column);
+			gtk_object_sink (GTK_OBJECT (view->details->file_name_column));
 			view->details->file_name_column_num = column_num;
 			
 			g_hash_table_insert (view->details->columns,
@@ -1295,7 +1326,6 @@ create_and_set_up_tree_view (FMListView *view)
 			gtk_tree_view_column_set_cell_data_func (view->details->file_name_column, cell,
 								 (GtkTreeCellDataFunc) filename_cell_data_func,
 								 view, NULL);
-			gtk_tree_view_append_column (view->details->tree_view, view->details->file_name_column);
 		} else {		
 			cell = gtk_cell_renderer_text_new ();
 			g_object_set (cell, "xalign", xalign, NULL);
@@ -1305,6 +1335,8 @@ create_and_set_up_tree_view (FMListView *view)
 									   cell,
 									   "text", column_num,
 									   NULL);
+			g_object_ref (column);
+			gtk_object_sink (GTK_OBJECT (column));
 			gtk_tree_view_column_set_sort_column_id (column, column_num);
 			g_hash_table_insert (view->details->columns, 
 					     g_strdup (name), 
@@ -1312,7 +1344,6 @@ create_and_set_up_tree_view (FMListView *view)
 			
 			gtk_tree_view_column_set_resizable (column, TRUE);
 			gtk_tree_view_column_set_visible (column, TRUE);
-			gtk_tree_view_append_column (view->details->tree_view, column);
 		}
 		g_free (name);
 		g_free (label);
@@ -1322,8 +1353,7 @@ create_and_set_up_tree_view (FMListView *view)
 	/* Apply the default column order and visible columns, to get it
 	 * right most of the time. The metadata will be checked when a 
 	 * folder is loaded */
-	apply_visible_columns (view, default_visible_columns_auto_value);
-	apply_column_order (view, default_column_order_auto_value);
+	apply_columns_settings (view, default_column_order_auto_value, default_visible_columns_auto_value);
 
 	gtk_widget_show (GTK_WIDGET (view->details->tree_view));
 	gtk_container_add (GTK_CONTAINER (view), GTK_WIDGET (view->details->tree_view));
@@ -1362,16 +1392,6 @@ get_visible_columns (FMListView *list_view)
 	return visible_columns;
 }
 
-static void
-set_visible_columns_from_metadata_and_preferences (FMListView *list_view)
-{
-	GList *visible_columns;
-
-	visible_columns = get_visible_columns (list_view);
-	apply_visible_columns (list_view, visible_columns);
-	eel_g_list_free_deep (visible_columns);
-}
-
 static GList *
 get_column_order (FMListView *list_view)
 {
@@ -1393,13 +1413,18 @@ get_column_order (FMListView *list_view)
 }
 
 static void
-set_column_order_from_metadata_and_preferences (FMListView *list_view)
+set_columns_settings_from_metadata_and_preferences (FMListView *list_view)
 {
 	GList *column_order;
+	GList *visible_columns;
 
 	column_order = get_column_order (list_view);
-	apply_column_order (list_view, column_order);
+	visible_columns = get_visible_columns (list_view);
+
+	apply_columns_settings (list_view, column_order, visible_columns);
+
 	eel_g_list_free_deep (column_order);
+	eel_g_list_free_deep (visible_columns);
 }
 
 static void
@@ -1481,8 +1506,7 @@ fm_list_view_begin_loading (FMDirectoryView *view)
 
 	set_sort_order_from_metadata_and_preferences (list_view);
 	set_zoom_level_from_metadata_and_preferences (list_view);
-	set_visible_columns_from_metadata_and_preferences (list_view);
-	set_column_order_from_metadata_and_preferences (list_view);
+	set_columns_settings_from_metadata_and_preferences (list_view);
 }
 
 static void
@@ -1515,10 +1539,59 @@ fm_list_view_clear (FMDirectoryView *view)
 	}
 }
 
+static void
+fm_list_view_rename_callback (NautilusFile *file, GnomeVFSResult result, gpointer callback_data)
+{
+	FMListView *view;
+	
+	view = FM_LIST_VIEW (callback_data);
+
+	if (view->details->renaming_file) {
+		view->details->rename_done = TRUE;
+		
+		if (result != GNOME_VFS_OK) {
+			/* If the rename failed (or was cancelled), kill renaming_file.
+			 * We won't get a change event for the rename, so otherwise
+			 * it would stay around forever.
+			 */
+			nautilus_file_unref (view->details->renaming_file);
+			view->details->renaming_file = NULL;
+		}
+	}
+	
+	g_object_unref (view);
+}
+
+
 static void
 fm_list_view_file_changed (FMDirectoryView *view, NautilusFile *file, NautilusDirectory *directory)
 {
-	fm_list_model_file_changed (FM_LIST_VIEW (view)->details->model, file, directory);
+	FMListView *listview;
+	GtkTreeIter iter;
+	GtkTreePath *file_path;
+
+	listview = FM_LIST_VIEW (view);
+	
+	fm_list_model_file_changed (listview->details->model, file, directory);
+
+	if (listview->details->renaming_file != NULL &&
+	    file == listview->details->renaming_file &&
+	    listview->details->rename_done) {
+		/* This is (probably) the result of the rename operation, and
+		 * the tree-view changes above could have resorted the list, so
+		 * scroll to the new position
+		 */
+		if (fm_list_model_get_tree_iter_from_file (listview->details->model, file, directory, &iter)) {
+			file_path = gtk_tree_model_get_path (GTK_TREE_MODEL (listview->details->model), &iter);
+			gtk_tree_view_scroll_to_cell (listview->details->tree_view,
+						      file_path, NULL,
+						      FALSE, 0.0, 0.0);
+			gtk_tree_path_free (file_path);
+		}
+		
+		nautilus_file_unref (listview->details->renaming_file);
+		listview->details->renaming_file = NULL;
+	}
 }
 
 static GtkWidget *
@@ -1790,8 +1863,7 @@ column_chooser_changed_callback (NautilusColumnChooser *chooser,
 	eel_g_list_free_deep (visible_columns);
 	eel_g_list_free_deep (column_order);
 
-	set_visible_columns_from_metadata_and_preferences (view);
-	set_column_order_from_metadata_and_preferences (view);
+	set_columns_settings_from_metadata_and_preferences (view);
 }
 
 static void
@@ -1830,8 +1902,7 @@ column_chooser_use_default_callback (NautilusColumnChooser *chooser,
 	nautilus_file_set_metadata_list (file, NAUTILUS_METADATA_KEY_LIST_VIEW_COLUMN_ORDER, NAUTILUS_METADATA_SUBKEY_COLUMNS, NULL);
 	nautilus_file_set_metadata_list (file, NAUTILUS_METADATA_KEY_LIST_VIEW_VISIBLE_COLUMNS, NAUTILUS_METADATA_SUBKEY_COLUMNS, NULL);
 
-	set_visible_columns_from_metadata_and_preferences (FM_LIST_VIEW (view));
-	set_column_order_from_metadata_and_preferences (FM_LIST_VIEW (view));
+	set_columns_settings_from_metadata_and_preferences (FM_LIST_VIEW (view));
 	column_chooser_set_from_settings (chooser, view);
 }
 
@@ -1979,8 +2050,7 @@ fm_list_view_reset_to_defaults (FMDirectoryView *view)
 		 default_sort_reversed_auto_value ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);
 
 	fm_list_view_set_zoom_level (FM_LIST_VIEW (view), get_default_zoom_level (), FALSE);
-	set_visible_columns_from_metadata_and_preferences (FM_LIST_VIEW (view));
-	set_column_order_from_metadata_and_preferences (FM_LIST_VIEW (view));
+	set_columns_settings_from_metadata_and_preferences (FM_LIST_VIEW (view));
 }
 
 static void
@@ -2211,10 +2281,10 @@ fm_list_view_click_policy_changed (FMDirectoryView *directory_view)
 	GtkTreeIter iter;
 	GtkTreeView *tree;
 
+	view = FM_LIST_VIEW (directory_view);
+
 	/* ensure that we unset the hand cursor and refresh underlined rows */
 	if (click_policy_auto_value == NAUTILUS_CLICK_POLICY_DOUBLE) {
-		view = FM_LIST_VIEW (directory_view);
-		
 		if (view->details->hover_path != NULL) {
 			if (gtk_tree_model_get_iter (GTK_TREE_MODEL (view->details->model),
 						     &iter, view->details->hover_path)) {
@@ -2236,7 +2306,15 @@ fm_list_view_click_policy_changed (FMDirectoryView *directory_view)
 				gdk_display_flush (display);
 			}
 		}
-		
+
+		if (hand_cursor != NULL) {
+			gdk_cursor_unref (hand_cursor);
+			hand_cursor = NULL;
+		}
+	} else if (click_policy_auto_value == NAUTILUS_CLICK_POLICY_SINGLE) {
+		if (hand_cursor == NULL) {
+			hand_cursor = gdk_cursor_new(GDK_HAND2);
+		}
 	}
 }
 
@@ -2279,8 +2357,8 @@ default_visible_columns_changed_callback (gpointer callback_data)
 	FMListView *list_view;
 	
 	list_view = FM_LIST_VIEW (callback_data);
-	
-	set_visible_columns_from_metadata_and_preferences (list_view);
+
+	set_columns_settings_from_metadata_and_preferences (list_view);	
 }
 
 static void
@@ -2289,8 +2367,8 @@ default_column_order_changed_callback (gpointer callback_data)
 	FMListView *list_view;
 	
 	list_view = FM_LIST_VIEW (callback_data);
-	
-	set_column_order_from_metadata_and_preferences (list_view);
+
+	set_columns_settings_from_metadata_and_preferences (list_view);
 }
 
 static void
@@ -2340,6 +2418,11 @@ fm_list_view_dispose (GObject *object)
 		list_view->details->drag_dest = NULL;
 	}
 
+	if (list_view->details->renaming_file_activate_timeout != 0) {
+		g_source_remove (list_view->details->renaming_file_activate_timeout);
+		list_view->details->renaming_file_activate_timeout = 0;
+	}
+
 	ui_manager = fm_directory_view_get_ui_manager (FM_DIRECTORY_VIEW (list_view));
 	if (ui_manager != NULL) {
 		nautilus_ui_unmerge_ui (ui_manager,
@@ -2375,6 +2458,10 @@ fm_list_view_finalize (GObject *object)
 	g_list_free (list_view->details->cells);
 	g_hash_table_destroy (list_view->details->columns);
 
+	if (list_view->details->hover_path != NULL) {
+		gtk_tree_path_free (list_view->details->hover_path);
+	}
+
 	g_free (list_view->details);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -2581,6 +2668,8 @@ fm_list_view_init (FMListView *list_view)
 	
 	/* ensure that the zoom level is always set in begin_loading */
 	list_view->details->zoom_level = NAUTILUS_ZOOM_LEVEL_SMALLEST - 1;
+
+	list_view->details->hover_path = NULL;
 }
 
 static NautilusView *
